% CSTR model from
%
%  H. Chen, A. Kremling, and F. Allgower.  Nonlinear predictive
%  control of a benchmark CSTR.  In Proceedings of the 3rd European
%  Control Conference ECC'95, pages 3247--3252, Rome, Italy, 1995.

%  Description:  CSTR with cooling jacket dynamics.  I think I need to get that paper - the units don't look SI.
%    I think VoVr is Q/V where Q is the volumetric flowrate and V is the volume of the jacket.

function xdot=cstr2(x,t)
% Note that the reference is wrong; the volume is .01 L, not 10 L.  Otherwise,
% the steady states reported are not possible.

global  u
% x = [Ca; Cb; T; Tk]
% u = [VoVr; Qk]
    
% Inputs (2):
VoVr = u(1);
Qk = u(2);

% States (4):
Ca = x(1);
Cb = x(2);
T = x(3);
Tk = x(4);

% Parameters:
Ca0 =  5.1;
T0 = 104.9;

k10 = 1.287e12;
k20 = 1.287e12;
k30 = 9.043e9;
E1 = -9758.3;
E2 = -9758.3;
E3 = -8560;
dHrAB = 4.2;
dHrBC = -11;
dHrAD = -41.85;
rho = 0.9342;
Cp = 3.01;
kw = 4032;
AR = .215; 
VR = .01;
mK = 5;
CpK = 2;

k1 = k10*exp(E1/(T+273.15));
k2 = k20*exp(E2/(T+273.15));
k3 = k30*exp(E3/(T+273.15));

xdot(1) = VoVr*(Ca0 - Ca) - k1*Ca - k3*Ca^2;
xdot(2) = -VoVr*Cb + k1*Ca - k2*Cb;
xdot(3) = VoVr*(T0 - T) - (1/(rho*Cp))*(k1*Ca*dHrAB + k2*Cb*dHrBC + k3*Ca^2*dHrAD) + kw*AR*(Tk - T)/(rho*Cp*1e3*VR);
xdot(4) = (1/(mK*CpK))*(Qk + kw*AR*(T - Tk));
