% CSTR model from
%
%  H. Chen, A. Kremling, and F. Allgower.  Nonlinear predictive
%  control of a benchmark CSTR.  In Proceedings of the 3rd European
%  Control Conference ECC'95, pages 3247--3252, Rome, Italy, 1995.


function jac=cstr4partials(x)

  global uk 
  u = uk;

  A = zeros(4,4);
  B = zeros(4,2);

  % x = [Ca-Caset; Cb-Cbset; T-Tset; Tk-Tkset]
  % u = [VoVr-VoVrset; Qk-Qkset]

  Ca = x(1);
  Cb = x(2);
  T = x(3);
  Tk = x(4);
  VoVr = u(1);
  Qk = u(2);

  Ca0 =  5.1;
  T0 = 104.9;

  k10 = 1.287e12;
  k20 = 1.287e12;
  k30 = 9.043e9;
  E1 = -9758.3;
  E2 = -9758.3;
  E3 = -8560;
  dHrAB = 4.2;
  dHrBC = -11;
  dHrAD = -41.85;
  rho = 0.9342;
  Cp = 3.01;
  kw = 4032;
  AR = .215; 
  VR = .01;
  mK = 5;
  CpK = 2;

  k1 = k10*exp(E1/(T+273.15));
  k2 = k20*exp(E2/(T+273.15));
  k3 = k30*exp(E3/(T+273.15));
  dk1 = k1*E1/(T+273.15)^2;
  dk2 = k2*E2/(T+273.15)^2;
  dk3 = k3*E3/(T+273.15)^2;

  A(1,1) = -VoVr-k1-2*k3*Ca;
  A(1,2) = 0;
  A(1,3) = dk1*Ca+dk3*Ca^2;
  A(1,4) = 0;
  B(1,1) = Ca0-Ca;
  B(1,2) = 0;
  A(2,1) = k1;
  A(2,2) = -VoVr-k2;
  A(2,3) = -dk1*Ca+dk2*Cb;
  A(2,4) = 0;
  B(2,1) = -Cb;
  B(2,2) = 0;
  A(3,1) = (-1/(rho*Cp))*(k1*dHrAB+2*k3*Ca*dHrAD);
  A(3,2) = (-1/(rho*Cp))*(k2*dHrBC);
  A(3,3) = -VoVr -(1/(rho*Cp))*(-dk1*Ca*dHrAB-dk2*Cb*dHrBC- ...
                    dk3*Ca^2*dHrAD) - kw*AR/(rho*Cp*1e3*VR);
  A(3,4) = kw*AR/(rho*Cp*1e3*VR);
  B(3,1) = T0-T;
  B(3,2) = 0;
  A(4,1) = 0;
  A(4,2) = 0;
  A(4,3) = kw*AR/(mK*CpK);
  A(4,4) = -kw*AR/(mK*CpK);
  B(4,1) = 0;
  B(4,2) = 1/(mK*CpK);

  jac = [A B];

endfunction
